﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.SessionState;
using System.Collections.Specialized;
using System.Web.Hosting;

namespace HttpContextTool
{
    /// <summary> 
    ///   Provides helper methods for initalizing an <see cref="HttpContext"/> object when simulating ASP.NET request
    ///   cycles. 
    /// </summary>
    public class HttpContextHelper
    {
        public static readonly string s_appVirtualDir = "/";
        public static readonly string s_appPhysicalDir = @"c:\";
        public static readonly string s_serverPath = "http://127.0.0.1";

        public static void SetCurrent(HttpContext context)
        {
            HttpContext.Current = context;
        }

        public static HttpContext CreateHttpContext(string httpMethod, string page, string query)
        {
            SimpleWorkerRequest workerRequest =
                new SimpleWorkerRequest(s_appVirtualDir, s_appPhysicalDir, page, query, new System.IO.StringWriter());

            object httpRuntime = PrivateInvoke.GetNonPublicStaticField(typeof(HttpRuntime), "_theRuntime");
            PrivateInvoke.SetNonPublicField(httpRuntime, "_appDomainAppPath", s_appPhysicalDir);
            string assemblyName = typeof(HttpApplication).Assembly.FullName;
            Type virtualPathType = Type.GetType("System.Web.VirtualPath, " + assemblyName, true);
            object virtualPath = PrivateInvoke.InvokePublicStaticMethod(virtualPathType, "Create", s_appVirtualDir);
            PrivateInvoke.SetNonPublicField(httpRuntime, "_appDomainAppVPath", virtualPath);
            PrivateInvoke.SetNonPublicField(httpRuntime, "_appDomainAppId", "Rubicon.Web.UnitTests");
            Type buildManagerType = typeof(System.Web.Compilation.BuildManager);
            PrivateInvoke.SetNonPublicStaticProperty(buildManagerType, "SkipTopLevelCompilationExceptions", true);
            HttpContext context = new HttpContext(workerRequest);
            PrivateInvoke.SetNonPublicField(context.Request, "_httpMethod", httpMethod);

            HttpSessionState sessionState = CreateSession();
            SetSession(context, sessionState);

            context.Request.Browser = new HttpBrowserCapabilities();

            return context;
        }

        public static void SetQueryString(HttpContext context, NameValueCollection queryString)
        {
            PrivateInvoke.InvokeNonPublicMethod(context.Request.QueryString, "MakeReadWrite", new object[0]);
            context.Request.QueryString.Clear();
            foreach (string key in queryString)
                context.Request.QueryString.Set(key, queryString[key]);
            PrivateInvoke.InvokeNonPublicMethod(context.Request.QueryString, "MakeReadOnly", new object[0]);

            PrivateInvoke.SetNonPublicField(context.Request, "_params", null);
        }

        public static void SetForm(HttpContext context, NameValueCollection form)
        {
            PrivateInvoke.InvokeNonPublicMethod(context.Request.Form, "MakeReadWrite", new object[0]);
            context.Request.Form.Clear();
            foreach (string key in form)
                context.Request.Form.Set(key, form[key]);
            PrivateInvoke.InvokeNonPublicMethod(context.Request.Form, "MakeReadOnly", new object[0]);

            PrivateInvoke.SetNonPublicField(context.Request, "_params", null);
        }


        public static void SetFile(HttpContext context, string formName, string contentType, string fileName, byte[] file)
        {
            Type httpInputStream = typeof(HttpPostedFile).GetConstructors(System.Reflection.BindingFlags.NonPublic | System.Reflection.BindingFlags.Instance)[0].GetParameters()[2].ParameterType;

            Type httpRawUploadedContent = httpInputStream.GetConstructors(System.Reflection.BindingFlags.NonPublic | System.Reflection.BindingFlags.Instance)[0].GetParameters()[0].ParameterType;

            var row = PrivateInvoke.CreateInstanceNonPublicCtor(httpRawUploadedContent, new object[] { file.Length, file.Length });
            PrivateInvoke.SetNonPublicField(row, "_data", file);
            PrivateInvoke.SetNonPublicField(row, "_length", file.Length);
            PrivateInvoke.SetNonPublicField(row, "_completed", true);

            var stream = PrivateInvoke.CreateInstanceNonPublicCtor(httpInputStream, new object[] { row, 0, file.Length });

            HttpPostedFile f = (HttpPostedFile)PrivateInvoke.CreateInstanceNonPublicCtor(typeof(HttpPostedFile), new object[] { fileName, contentType, stream });

            PrivateInvoke.InvokeNonPublicMethod(context.Request.Files, "AddFile", new object[] { formName, f });
        }

        public static void SetServerVar(HttpContext context, NameValueCollection vars)
        {
            System.Collections.Hashtable hash = new System.Collections.Hashtable();

            Type entryType = typeof(System.Collections.Specialized.NameObjectCollectionBase).GetMethod("FindEntry", System.Reflection.BindingFlags.NonPublic | System.Reflection.BindingFlags.Instance).ReturnType;
            Type p = typeof(System.Web.HttpApplication).Assembly.GetType("System.Web.HttpServerVarsCollectionEntry");

            foreach (string key in vars)
            {
                var httpServerVarsCollectionEntry = PrivateInvoke.CreateInstanceNonPublicCtor(p, new object[] { key, vars[key] });

                var entry = PrivateInvoke.CreateInstanceNonPublicCtor(entryType, new object[] { key, httpServerVarsCollectionEntry });

                hash[key] = entry;
            }


            PrivateInvoke.SetNonPublicField(context.Request.ServerVariables, "_entriesTable", hash);
            PrivateInvoke.SetNonPublicField(context.Request.ServerVariables, "_allKeys", new string[] { "server_name" });
        }

        public static HttpSessionState CreateSession()
        {
            HttpSessionState sessionState;
            string id = Guid.NewGuid().ToString();
            HttpStaticObjectsCollection staticObjects = new HttpStaticObjectsCollection();
            int timeout = 20;
            bool newSession = true;
            SessionStateMode mode = SessionStateMode.InProc;
            bool isReadOnly = false;
            SessionStateItemCollection sessionItems = new SessionStateItemCollection();
            HttpSessionStateContainer httpSessionStateContainer = new HttpSessionStateContainer(
                id, sessionItems, staticObjects, timeout, newSession, HttpCookieMode.UseUri, mode, isReadOnly);

            sessionState = (HttpSessionState)PrivateInvoke.CreateInstanceNonPublicCtor(typeof(HttpSessionState), httpSessionStateContainer);
            return sessionState;
        }

        public static void SetSession(HttpContext context, HttpSessionState sessionState)
        {
            context.Items["AspSession"] = sessionState;
        }

        private HttpContextHelper()
        {
        }
    }
}
